# -*- coding: utf-8 -*-
"""
:mod:`signal` -- This provides handy signal utilities.
======================================================

.. module:: evasion.common.signal
   :platform: Unix, MacOSX, Windows
   :synopsis: This provides handy utiliy functions for network services.
.. moduleauthor:: Oisin Mulvihill <oisin.mulvihill@gmail.com>

.. autoclass:: evasion.common.signal.CallBack

"""
import threading


class WaitTimeout(Exception):
    """Raised by CallBack.wait if it timeout before receiving a call."""


class CallBack(object):
    """Helpful message catcher to allow testing of asynchronous messages.

    self.data will contain the captured data.

    """
    def __init__(self, timeout=180):
        self.waiter = threading.Event()
        self.data = None
        self.timeout = int(timeout)

    def wait(self):
        """Wait for self.timeout seconds or raise ValueError on timeout."""
        self.waiter.wait(self.timeout)
        if not self.data:
            raise WaitTimeout("The callback never called before timeout.")
        # reset for next run:
        self.waiter.clear()

    def __call__(self, data):
        """Callback invoked, store given data as self.data."""
        self.data = data
        self.waiter.set()




# -*- coding: utf-8 -*-
"""
"""
import socket
import logging
import unittest

from evasion.common import log
from evasion.common import net
from evasion.common import signal
from evasion.common import webhelpers



class CommonTC(unittest.TestCase):


    def setUp(self):
        self.stop_needed = []

    def tearDown(self):
        [i.stop() for i in self.stop_needed]


    def testLog(self):
        """Calling log.to_console() shouldn't raise errors."""
        l = log.to_console()
        self.assertTrue(isinstance(l, logging.Logger))


    def testSignal(self):
        """Test the helper code provided by signal.
        """
        c = signal.CallBack(timeout=1)

        # Test the timeout waiting raises WaitTimeout
        self.assertRaises(signal.WaitTimeout, c.wait)
        self.assertEquals(c.data, None)

        # Test the timeout doesn't occur when the callback
        # is invoked.
        c(1)
        c.wait()
        self.assertEquals(c.data, 1)


    def testWaitForReadyLive(self):
        """Test wait for ready against a basic local web service.
        """
        # Get a free port to use for this test:
        port1 = net.get_free_port()

        web = webhelpers.BasicWeb(port=port1)
        self.stop_needed.append(web)

        # Run the web app and wait for ready should connect:
        web.start()
        result = net.wait_for_ready(web.uri)
        self.assertEquals(result, True)


    def testWaitForServiceLive(self):
        """Test wait for service against a basic local web service.
        """
        # Get a free port to use for this test:
        port1 = net.get_free_port()

        web = webhelpers.BasicWeb(port=port1)
        self.stop_needed.append(web)

        # Run the web app and wait for ready should connect:
        web.start()
        result = net.wait_for_service('localhost', port1)
        self.assertEquals(result, True)

        result = net.wait_for_service('localhost', port1, retries=1)
        self.assertEquals(result, True)


    def testNetTools(self):
        """Test as much of the net helper tools as I can.
        """
        # Recover a free port to construct a uri for a ficticious web servce
        # we can test the fail path of the wait_for_ready with:
        port = net.get_free_port()

        class XYZ(object):
            def __init__(self, port):
                self.port = port
            def freeport_range(self):
                return self.port

        fpr = XYZ(port)

        gport = net.get_free_port(fp=fpr.freeport_range)
        self.assertEquals(gport, port)

        # use the port
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.bind(('localhost', port))

        # Now exlucde this port and try again.
        self.assertRaises(net.NoFreePort, net.get_free_port, exclude_ports=[port], fp=fpr.freeport_range)

        s.close()

        # Recover a free port to construct a uri for a ficticious web servce
        # we can test the fail path of the wait_for_ready with:
        port = net.get_free_port()
        ficticious_uri = "http://localhost:%d" % port

        # Use this made up uri and see it did indeed NOT manage to connect:
        result = net.wait_for_ready(ficticious_uri, retries=1)
        self.assertEquals(result, False)

        # Use this made up uri and see it did indeed NOT manage to connect:
        result = net.wait_for_service('localhost', port, retries=1)
        self.assertEquals(result, False)


# -*- coding: utf-8 -*-
"""
:mod:`net` -- This provides handy utiliy functions for network services.
==============================================================================

.. module:: evasion.common.net
   :platform: Unix, MacOSX, Windows
   :synopsis: This provides handy utiliy functions for network services.
.. moduleauthor:: Oisin Mulvihill <oisin.mulvihill@gmail.com>

.. data:: PORT_RETRIES the amount of times get_free_port and wait_for_ready
will retry for by default.

.. autoexception:: evasion.common.net.NoFreePort

.. autofunction:: evasion.common.net.get_free_port

.. autofunction:: evasion.common.net.wait_for_service

.. autofunction:: evasion.common.net.wait_for_ready

"""
import time
import socket
import urllib
import random
import logging
import httplib
import urlparse


def get_log():
    return logging.getLogger('evasion.common.net')


PORT_RETRIES = 40


class NoFreePort(Exception):
    """Raised when get_free_port was unable to find a free port to use."""


def free_port_range(start=2000, stop=10000):
    """Used by get_free_port() to generate a random TCP port to test.

    :param start: The start of the TCP port range (default: 2000).

    :param stop: The stop of the TCP port range (default: 10000).

    :returns: A random int in the range(start, stop)

    """
    return random.randint(start, stop)


class NoFreePort(Exception):
    """Raised by get_free_port when no available TCP port could be found."""


def get_free_port(exclude_ports=[], retries=PORT_RETRIES, fp=free_port_range):
    """Called to return a free TCP port that we can use.

    This function gets a random port between 2000 - 10000.
    A test is done to check if the port is free by attempting
    to use it. If its not free another port is checked

    :exclude_ports: This is a list of port numbers to
    exclude from using. This could be a list of numbers
    generated by previous calls to this function.

    :param retries: The amount of attempts to try finding
    a free port.

    :param fp: The free port range number generator.

    This returns a TCP port number.

    :returns: The free port number to use.

    """
    log = get_log()
    returned = 0
    free_port = 0

    while retries:
        retries -= 1
        free_port = fp()

        # Get a port thats not in the exclude list
        exclude_retries = len(exclude_ports)
        while not exclude_retries:
            free_port = fp()
            if free_port in exclude_ports:
                exclude_retries -= 1
            else:
                break

        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            s.bind(('', free_port))
            s.close()

        except socket.error:
            # port not free, retry.
            log.info(
                "getFreePort: port not free %s, retrying with another port." % (
                free_port
            ))

        else:
            returned = free_port

    if not returned:
        # Retries finished and no free port was found:
        raise NoFreePort("I can't get a free port after retrying!")

    log.info("getFreePort: Free Port %s." % returned)

    return returned


def wait_for_service(host, port, retries=0, retry_period=5.0):
    """Called to wait until a socket connection can be made to a remote service.

    :param host: IP/DNS of the service to connect to.

    :param port: TCP port number to connect to.

    :param retries: (default: 0)

    The number of attempts before giving up on connecting
    to the browser. If this is 0 then we will wait forever
    for the browser to appear.

    :param retry_period: (default: 5.0) The amount of seconds
    to wait before the next connection attempt.

    :returns:

        True: success
        Failed: failed to connect after maximum retries.

    """
    log = get_log()
    returned = False
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    retries = int(retries)
    retry_period = float(retry_period)

    def check():
        returned = False
        try:
            log.debug("wait_for_service: connecting to host<%s> port<%s>"% (
                host,
                port
            ))
            s.connect((host, port))
        except socket.error:
            pass # not ready yet.
        else:
            log.debug("wait_for_service: Success! Connected to <%s:%s>"% (
                host,
                port
            ))
            returned = True

        try:
            s.close()
        except:
            pass

        return returned

    if not retries:
        # Keep connecting until its present:
        while True:
            time.sleep(retry_period)
            is_connected = check()
            if is_connected:
                returned = is_connected
                break
    else:
        # Give up after 'retries' attempts:
        for i in range(0, retries):
            returned = check()
            if returned:
                # Success!
                break
            else:
                time.sleep(retry_period)

    return returned


def wait_for_ready(uri, retries=PORT_RETRIES):
    """Called to wait for a web application to respond to normal requests.

    This function will attempt a HEAD request if its
    supported, otherwise it will use GET.

    :param uri: the URI of the web application on which
    it will receive requests.

    :param retries: The amount of attempts to try finding
    a free port.

    :returns: True: the web app ready.

    """
    returned = False

    URI = uri
    # Work set up the connection for the HEAD request:
    o = urlparse.urlsplit(URI)
    conn = httplib.HTTPConnection(o.hostname, o.port)

    while retries:
        retries -= 1
        try:
            # Just get the headers and not the body to speed things up.
            conn.request("HEAD",'/')
            res = conn.getresponse()
            if res.status == httplib.OK:
                # success, its ready.
                returned = True
                break;

            elif res.status == httplib.NOT_IMPLEMENTED:
                # HEAD not supported try a GET instead:
                try:
                    urllib.urlopen(URI)
                except IOError:
                    # Not ready yet. I should check the exception to
                    # make sure its socket error or we could be looping
                    # forever. I'll need to use a state machine if this
                    # prototype works. For now I'm taking the "head in
                    # the sand" approach.
                    pass
                else:
                    # success, its ready.
                    returned = True
                    break;

        except httplib.CannotSendRequest:
            # Not ready yet.
            pass

        except socket.error:
            # Not ready yet. I should check the exception to
            # make sure its socket error or we could be looping
            # forever. I'll need to use a state machine if this
            # prototype works. For now I'm taking the "head in
            # the sand" approach.
            pass

        time.sleep(0.8)

    return returned



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Provides helper logging set up functions.

"""
import logging


def to_console(default_log_level=logging.DEBUG):
    """Returns a logger configured for console outputs.

    :param default_log_level: logging.DEBUG by default.

    :returns: A logging.getLogger() root logger instance.

     """
    log = logging.getLogger()
    hdlr = logging.StreamHandler()
    fmt = '%(asctime)s %(name)s %(levelname)s %(message)s'
    formatter = logging.Formatter(fmt)
    hdlr.setFormatter(formatter)
    log.addHandler(hdlr)
    log.setLevel(logging.DEBUG)
    log.propagate = False

    return log


"""
Oisin Mulvihill
2012-01-19
"""
from setuptools import setup, find_packages

Name='evasion-common'
ProjectUrl="http://github.com/oisinmulvihill/evasion-common/tarball/master#egg=evasion_common"
Version='1.0.1'
Author='Oisin Mulvihill'
AuthorEmail='oisinmulvihill at gmail dot com'
Maintainer=' Oisin Mulvihill'
Summary='Helper functions collected together from other evasion modules to aid reuse.'
License='Evasion Project CDDL License'
ShortDescription=Summary

try:
    fd = open("README.rst","r")
    Description = fd.read()
    fd.close()
except IOError:
    Description=Summary


needed = [
]

EagerResources = [
    'evasion',
]

ProjectScripts = [
]

PackageData = {
    '': ['*.*'],
}

EntryPoints = """
"""


setup(
    url=ProjectUrl,
    name=Name,
    zip_safe=False,
    version=Version,
    author=Author,
    author_email=AuthorEmail,
    description=ShortDescription,
    long_description=Description,
    license=License,
    scripts=ProjectScripts,
    install_requires=needed,
    setup_requires=[
      'nose>=1.0.0',
      'coverage'
    ],
    test_suite="nose.collector",
    entry_points=EntryPoints,
    packages=find_packages('lib'),
    package_data=PackageData,
    package_dir = {'': 'lib'},
    eager_resources = EagerResources,
    namespace_packages = ['evasion'],
)



# -*- coding: utf-8 -*-
"""
This providers some helper webservice classes that can be built on for testing
or generally elsewhere.

"""
import time
import thread
import socket
import logging
import SocketServer
import SimpleHTTPServer



class StoppableTCPServer(SocketServer.TCPServer):
    """Handle requests but check for the exit flag setting periodically.
    """
    log = logging.getLogger('evasion')

    exitTime = False

    allow_reuse_address = True

    def __init__(self, serveraddress, ControlFrameRequest):
        SocketServer.TCPServer.__init__(self, serveraddress, ControlFrameRequest)

    def stop(self):
        self.exitTime = True
        self.log.info("Stop: set exit flag and closed port.")

    def server_bind(self):
        SocketServer.TCPServer.server_bind(self)
        self.socket.settimeout(1)
        self.run = True

    def get_request(self):
        """Handle a request/timeout and check the exit flag.
        """
        while not self.exitTime:
            try:
                returned = self.socket.accept()
                if len(returned) > 1:
                    conn, address = returned
                    conn.settimeout(None)
                    returned = (conn, address)
                return returned
            except socket.timeout:
                pass


class BasicWeb(object):
    """Implements a dump web server I can use in testing.
    """
    def __init__(self, port, interface='localhost'):
        super(BasicWeb, self).__init__()
        self.log = logging.getLogger("evasion")
        self.server = None
        self.port = int(port)
        self.interface = interface
        self.uri = "http://%s:%s" % (interface, port)


    def start(self):
        """Start the uber basic interface.

        All request are handled by BasicWeb method handle_request.

        """
        def make_handler(parent):
            class RequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
                # A super-simple HTTP request handler:
                def do_GET(self):
                    parent.handle_request('GET', self)
                def do_POST(self):
                    parent.handle_request('POST',self)
                def do_PUT(self):
                    parent.handle_request('PUT',self)
                def do_DELETE(self):
                    parent.handle_request('DELETE',self)
                def do_HEAD(self):
                    parent.handle_request('HEAD',self)

            return RequestHandler

        self.log.info("Creating FakeWebServer.")
        self.server = StoppableTCPServer(
            (self.interface, self.port),
            make_handler(self)
        )

        def _start(data=0):
            self.log.info("start: URI<%s>" % self.uri)
            self.server.serve_forever()

        thread.start_new(_start, (0,))


    def stop(self):
        """Stop the web server running.
        """
        if self.server:
            self.server.stop()


    def handle_request(self, method, req, content='OK', mime_type="text/plain"):
        """Return OK 200 response with text/plain OK in the response body.

        Override this method to implement other behaviour.

        """
        body = content
        req.send_response(200)
        req.send_header("Content-type", mime_type)
        req.send_header("Content-Length", len(body))
        last_modified = time.asctime()
        req.send_header("Last-Modified", last_modified)
        req.end_headers()
        req.wfile.write(body)


